--
-- Technology License for Research and Education Purposes
--
-- Washington University ("University") in St. Louis has
-- developed and is the owner of the copyright for Field-programmable
-- Port Extender (FPX) Technology.
--
-- The University grants to select not-for-profit educational institution
-- ("Licensee") receiving the Technology the right to use its Technology
-- solely for Licensee's internal nonprofit research and academic purposes.
-- This License is nontransferrable, royalty free and non-exclusive.
-- the licensee may reproduce Technology only for educational,
-- research, backup or archival purposes
--
-- UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
-- OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
-- AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
-- AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
-- OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
--
-- Licensee assumes the entire risk as to the performance of Technology and/or
-- associated materials, and to the performance and validity of information
-- generated using Technology.
--
-- For complete terms of the agreement, review FPX-license.txt,
-- which is included in this file distribution.
--
--
-- Washington University
-- Applied Research Laboratory
-- 1 Brookings Drive
-- Campus Box 1045
-- Saint Louis, MO 63130  USA
-- Phone: (314) 935-4460
-- Web: http://www.arl.wustl.edu/arl/projects/fpx/
--

library ieee;
use ieee.std_logic_1164.all;


entity app_module is
  port (
    -- clock & reset
    clk     : in std_logic;             -- 100mhz global clock
    reset_l : in std_logic;             -- synchronous reset, asserted-low

    -- enable & ready
    -- handshake for module reconfiguration.
    enable_l : in  std_logic;           -- asserted low
    ready_l  : out std_logic;           -- asserted low

    -- cell input interface
    soc_mod_in   : in  std_logic;                     -- start of cell
    d_mod_in     : in  std_logic_vector(31 downto 0); -- 32-bit data
    tca_mod_in   : out std_logic;                     -- transmit cell available

    -- cell output interface
    soc_out_mod   : out std_logic;                      -- start of cell
    d_out_mod     : out std_logic_vector(31 downto 0);  -- 32-bit data
    tca_out_mod   : in  std_logic);
end app_module;

architecture struc of app_module is

  component udpwrapper
    port (
      clk             : in  std_logic;    -- clock
      reset_l         : in  std_logic;    -- reset
      enable_l        : in  std_logic;    -- enable
      ready_l         : out std_logic;    -- ready
      soc_mod_in      : in  std_logic;    -- start of cell
      d_mod_in        : in  std_logic_vector (31 downto 0);  -- data
      tca_mod_in      : out std_logic;    -- transmit cell available
      d_out_appl      : out std_logic_vector (31 downto 0);  -- data to appl
      dataen_out_appl : out std_logic;    -- data enable
      sof_out_appl    : out std_logic;    -- start of frame
      soip_out_appl   : out std_logic;
      eof_out_appl    : out std_logic;    -- end of frame
      sod_out_appl    : out std_logic;    -- start of datagram
      tca_out_appl    : in  std_logic;    -- congestion control
      d_appl_in       : in  std_logic_vector (31 downto 0);  -- data from appl
      dataen_appl_in  : in  std_logic;    -- data enable
      sof_appl_in     : in  std_logic;    -- start of frame
      soip_appl_in    : in  std_logic;
      eof_appl_in     : in  std_logic;    -- end of frame
      sod_appl_in     : in  std_logic;    -- start of datagram
      tca_appl_in     : out std_logic;    -- congestion control
      soc_out_mod     : out std_logic;    -- start of cell
      d_out_mod       : out std_logic_vector (31 downto 0);  -- data
      tca_out_mod     : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- interface 
  -----------------------------------------------------------------------------

  component wrapper_app
    port (
      clk              : in  std_logic;                      -- clock
      reset_l          : in  std_logic;                      -- reset
      enable_l         : in std_logic;
      ready_l          : out std_logic;
      dataen_out_appl  : in  std_logic;                      -- data enable
      d_out_appl       : in  std_logic_vector (31 downto 0); -- data
      sof_out_appl     : in  std_logic;                      -- start of frame
      soip_out_appl    : in  std_logic;
      eof_out_appl     : in  std_logic;                      -- end of frame
      sod_out_appl     : in  std_logic;                      -- start of datagram
      tca_appl_in      : in  std_logic;                      -- congestion control
      dataen_appl_in   : out std_logic;                      -- data enable
      d_appl_in        : out std_logic_vector (31 downto 0); -- data
      sof_appl_in      : out std_logic;                      -- start of frame
      soip_appl_in     : out std_logic;
      eof_appl_in      : out std_logic;                      -- end of frame
      sod_appl_in      : out std_logic;                      -- start of datagram
      tca_out_appl     : out  std_logic);                    -- congestion control
  end component;

  signal data_u2a : std_logic_vector (31 downto 0);  -- data
  signal dataen_u2a : std_logic;                     -- data enable
  signal sof_u2a  : std_logic;                       -- start of frame
  signal soip_u2a : std_logic;
  signal eof_u2a  : std_logic;                       -- end of frame
  signal sop_u2a  : std_logic;                       -- start of payload
  signal tca_u2a  : std_logic;                       -- congestion control

  signal data_a2u : std_logic_vector (31 downto 0);  -- data
  signal dataen_a2u : std_logic;                     -- data enable
  signal sof_a2u  : std_logic;                       -- start of frame
  signal soip_a2u : std_logic;
  signal eof_a2u  : std_logic;                       -- end of frame
  signal sop_a2u  : std_logic;                       -- start of payload
  signal tca_a2u  : std_logic;                       -- congestion control
  signal ready1_l : std_logic;
  signal ready2_l : std_logic;

begin 

  -----------------------------------------------------------------------------
  -- udpwrapper 
  -----------------------------------------------------------------------------
  udpw : udpwrapper port map (
    clk             => clk,
    reset_l         => reset_l,
    enable_l        => enable_l,
    ready_l         => ready1_l,
    soc_mod_in      => soc_mod_in,
    d_mod_in        => d_mod_in,
    tca_mod_in      => tca_mod_in,
    soc_out_mod     => soc_out_mod,
    d_out_mod       => d_out_mod,
    tca_out_mod     => tca_out_mod,
    d_out_appl      => data_u2a,
    dataen_out_appl => dataen_u2a,
    sof_out_appl    => sof_u2a,
    soip_out_appl   => soip_u2a,
    eof_out_appl    => eof_u2a,
    sod_out_appl    => sop_u2a,
    tca_out_appl    => tca_u2a,
    d_appl_in       => data_a2u,
    dataen_appl_in  => dataen_a2u,
    sof_appl_in     => sof_a2u,
    soip_appl_in    => soip_a2u,
    eof_appl_in     => eof_a2u,
    sod_appl_in     => sop_a2u,
    tca_appl_in     => tca_a2u);


  -----------------------------------------------------------------------------
  -- application
  -----------------------------------------------------------------------------
  app : wrapper_app port map (
    clk              => clk,
    reset_l          => reset_l,
    enable_l         => enable_l,
    ready_l          => ready2_l,

    dataen_out_appl  => dataen_u2a,
    d_out_appl       => data_u2a,
    sof_out_appl     => sof_u2a,
    soip_out_appl    => soip_u2a,
    eof_out_appl     => eof_u2a,
    sod_out_appl     => sop_u2a,
    tca_appl_in      => tca_a2u,

    dataen_appl_in   => dataen_a2u,
    d_appl_in        => data_a2u,
    sof_appl_in      => sof_a2u,
    soip_appl_in     => soip_a2u,
    eof_appl_in      => eof_a2u,
    sod_appl_in      => sop_a2u,
    tca_out_appl     => tca_u2a);

  ---------------------------------
  -- TESTPINS 
  ---------------------------------
  -- testpins <= 

  ready_l <= ready1_l or ready2_l;

end struc;
